'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = createActionAsync;

var _reduxAct = require('redux-act');

var _lodash = require('lodash.defaults');

var _lodash2 = _interopRequireDefault(_lodash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var defaultOption = {
  request: {},
  ok: {},
  error: {}
};

function createActionAsync(description, api) {
  var options = arguments.length <= 2 || arguments[2] === undefined ? defaultOption : arguments[2];

  (0, _lodash2.default)(options, defaultOption);
  var actions = {
    request: (0, _reduxAct.createAction)(description + '_REQUEST', options.request.payloadReducer, options.request.metaReducer),
    ok: (0, _reduxAct.createAction)(description + '_OK', options.ok.payloadReducer, options.ok.metaReducer),
    error: (0, _reduxAct.createAction)(description + '_ERROR', options.error.payloadReducer, options.error.metaReducer)
  };

  var actionAsync = function actionAsync() {
    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return function (dispatch) {
      dispatch(actions.request.apply(actions, args));
      return api.apply(undefined, args).then(function (res) {
        dispatch(actions.ok(res));
      }).catch(function (err) {
        dispatch(actions.error(err));
        if (options.rethrow) throw err;
      });
    };
  };

  actionAsync.request = actions.request;
  actionAsync.ok = actions.ok;
  actionAsync.error = actions.error;
  return actionAsync;
};